Index: src/mips64/lithium-codegen-mips64.cc |
diff --git a/src/mips64/lithium-codegen-mips64.cc b/src/mips64/lithium-codegen-mips64.cc |
index a612681c5b37086779db837e4ec22f94da26eecd..80befa531dc93016f07ad858783ad08eff710e97 100644 |
--- a/src/mips64/lithium-codegen-mips64.cc |
+++ b/src/mips64/lithium-codegen-mips64.cc |
@@ -2648,30 +2648,20 @@ void LCodeGen::EmitClassOfTest(Label* is_true, |
__ JumpIfSmi(input, is_false); |
- if (String::Equals(isolate()->factory()->Function_string(), class_name)) { |
- // Assuming the following assertions, we can use the same compares to test |
- // for both being a function type and being in the object type range. |
- STATIC_ASSERT(NUM_OF_CALLABLE_SPEC_OBJECT_TYPES == 2); |
- STATIC_ASSERT(FIRST_NONCALLABLE_SPEC_OBJECT_TYPE == |
- FIRST_SPEC_OBJECT_TYPE + 1); |
- STATIC_ASSERT(LAST_NONCALLABLE_SPEC_OBJECT_TYPE == |
- LAST_SPEC_OBJECT_TYPE - 1); |
- STATIC_ASSERT(LAST_SPEC_OBJECT_TYPE == LAST_TYPE); |
+ STATIC_ASSERT(LAST_JS_RECEIVER_TYPE == LAST_TYPE); |
+ __ GetObjectType(input, temp, temp2); |
+ __ Branch(is_false, lt, temp2, Operand(FIRST_JS_RECEIVER_TYPE)); |
- __ GetObjectType(input, temp, temp2); |
- __ Branch(is_false, lt, temp2, Operand(FIRST_SPEC_OBJECT_TYPE)); |
- __ Branch(is_true, eq, temp2, Operand(FIRST_SPEC_OBJECT_TYPE)); |
- __ Branch(is_true, eq, temp2, Operand(LAST_SPEC_OBJECT_TYPE)); |
+ // Objects with [[Call]] have class 'Function'. |
+ __ lbu(temp2, FieldMemOperand(temp, Map::kBitFieldOffset)); |
+ __ And(temp2, temp2, Operand(1 << Map::kIsCallable)); |
+ if (String::Equals(isolate()->factory()->Function_string(), class_name)) { |
+ __ Branch(is_true, ne, temp2, Operand(zero_reg)); |
} else { |
- // Faster code path to avoid two compares: subtract lower bound from the |
- // actual type and do a signed compare with the width of the type range. |
- __ GetObjectType(input, temp, temp2); |
- __ Dsubu(temp2, temp2, Operand(FIRST_NONCALLABLE_SPEC_OBJECT_TYPE)); |
- __ Branch(is_false, gt, temp2, Operand(LAST_NONCALLABLE_SPEC_OBJECT_TYPE - |
- FIRST_NONCALLABLE_SPEC_OBJECT_TYPE)); |
+ __ Branch(is_false, ne, temp2, Operand(zero_reg)); |
} |
// Now we are in the FIRST-LAST_NONCALLABLE_SPEC_OBJECT_TYPE range. |
// Check if the constructor in the map is a function. |
Register instance_type = scratch1(); |
DCHECK(!instance_type.is(temp)); |
@@ -2648,30 +2648,20 @@ void LCodeGen::EmitClassOfTest(Label* is_true, |
} |
- // Now we are in the FIRST-LAST_NONCALLABLE_SPEC_OBJECT_TYPE range. |
+ // Now we are in the FIRST-LAST_JS_RECEIVER_TYPE range. |
// Check if the constructor in the map is a function. |
Register instance_type = scratch1(); |